@@ -32,7 +32,7 @@ import requests
 
 from .models import UserProfile
 from .http_client import HTTPClient
-from .exceptions import MojangError
+from .errors import MojangError
 
 log = logging.getLogger(__name__)
 
@@ -46,10 +46,11 @@ class API(HTTPClient):
     def __init__(
         self,
         session: Optional[requests.Session] = None,
-        ratelimit_sleep_duration: Optional[int] = 60,
+        retry_on_rate_limit: Optional[bool] = False,
+        ratelimit_sleep_time: Optional[int] = 60,
         debug_mode: Optional[bool] = False,
     ):
-        super().__init__(session, ratelimit_sleep_duration, debug_mode)
+        super().__init__(session, retry_on_rate_limit, ratelimit_sleep_time, debug_mode)
 
     def get_uuid(
         self,
@@ -89,14 +90,11 @@ class API(HTTPClient):
         else:
             url = f"{_API_BASE_URL}/users/profiles/minecraft/{username}"
 
-        resp = self._get(url)
-
-        if resp.status_code == 400:
-            return None
+        resp = self.request("get", url, ignore_codes=[400])
 
         try:
             return resp.json()["id"]
-        except json.decoder.JSONDecodeError:
+        except (KeyError, json.decoder.JSONDecodeError):
             return None
 
     def get_uuids(self, names: List[str]) -> Dict[str, str]:
@@ -128,13 +126,17 @@ class API(HTTPClient):
         if len(names) > 10:
             names = names[:10]
 
-        data = self._post(f"{_API_BASE_URL}/profiles/minecraft", json=names).json()
+        resp = self.request(
+            "post",
+            f"{_API_BASE_URL}/profiles/minecraft",
+            ignore_codes=[400],
+            json=names,
+        )
+
+        data = resp.json()
 
         if not isinstance(data, list):
-            if data.get("error"):
-                raise MojangError(data["errorMessage"])
-            else:
-                raise MojangError(data)
+            raise MojangError(response=resp)
 
         return {name_data["name"]: name_data["id"] for name_data in data}
 
@@ -155,7 +157,11 @@ class API(HTTPClient):
                 print("UUID does not appear to be valid.")
             ```
         """
-        resp = self._get(f"{_SESSIONSERVER_BASE_URL}/session/minecraft/profile/{uuid}")
+        resp = self.request(
+            "get",
+            f"{_SESSIONSERVER_BASE_URL}/session/minecraft/profile/{uuid}",
+            ignore_codes=[400],
+        )
 
         if resp.status_code == 400:
             return None
@@ -187,7 +193,11 @@ class API(HTTPClient):
                 # is_legacy_profile, and timestamp
             ```
         """
-        resp = self._get(f"{_SESSIONSERVER_BASE_URL}/session/minecraft/profile/{uuid}")
+        resp = self.request(
+            "get",
+            f"{_SESSIONSERVER_BASE_URL}/session/minecraft/profile/{uuid}",
+            ignore_codes=[400],
+        )
 
         try:
             value = resp.json()["properties"][0]["value"]
@@ -207,11 +217,11 @@ class API(HTTPClient):
             ```py
             servers = api.get_blocked_servers()
 
-            for hash in servers:
+            >> for hash in servers:
                 print(hash)
             ```
         """
-        resp = self._get(f"{_SESSIONSERVER_BASE_URL}/blockedservers")
+        resp = self.request("get", f"{_SESSIONSERVER_BASE_URL}/blockedservers")
         return resp.text.splitlines()
 
     def refresh_access_token(
@@ -253,7 +263,9 @@ class API(HTTPClient):
         }
 
         account = {}
-        data = self._post(f"{_AUTHSERVER_BASE_URL}/refresh", json=payload).json()
+        data = self.request(
+            "post", f"{_AUTHSERVER_BASE_URL}/refresh", json=payload
+        ).json()
 
         account["username"] = data["user"]["username"]
         account["uuid"] = data["user"]["id"]
