@@ -24,29 +24,31 @@ DEALINGS IN THE SOFTWARE.
 
 import ast
 import base64
+import json
+import logging
 from typing import Any, Dict, Optional
 import re
-import logging
-import json
 
 import requests
 
 from .http_client import HTTPClient
-from .exceptions import (
+from .models import UserProfile, Profile, Skin, Cape, NameInformation
+from .errors import (
     MojangError,
-    HTTPException,
-    LoginError,
+    BadRequest,
+    LoginFailure,
     MissingMinecraftLicense,
     MissingMinecraftProfile,
 )
-from .models import UserProfile, Profile, Skin, Cape, NameInformation
+
 
 log = logging.getLogger(__name__)
 
 
-BASE_API_URL = "https://api.minecraftservices.com"
+_BASE_API_URL = "https://api.minecraftservices.com"
 
-_XErrors = {
+# Potential Xbox Live login failure errors
+_XERRORS = {
     2148916233: "The account doesn't have an Xbox account.",
     2148916235: "The account is from a country where Xbox Live is not available/banned.",
     2148916236: "The account needs adult verification on Xbox page. (South Korea)",
@@ -62,10 +64,11 @@ class MojangAuth(HTTPClient):
         password: Optional[str] = None,
         bearer_token: Optional[str] = None,
         session: Optional[requests.Session] = None,
-        ratelimit_sleep_duration: Optional[int] = 60,
+        retry_on_rate_limit: Optional[bool] = False,
+        ratelimit_sleep_time: Optional[int] = 60,
         debug_mode: Optional[bool] = False,
     ):
-        super().__init__(session, ratelimit_sleep_duration, debug_mode)
+        super().__init__(session, retry_on_rate_limit, ratelimit_sleep_time, debug_mode)
 
         self.email = email
         self.password = password
@@ -83,10 +86,13 @@ class MojangAuth(HTTPClient):
         self._validate_session()
 
     def _validate_session(self):
-        resp = self._get(f"{BASE_API_URL}/entitlements/mcstore")
+        resp = self.request(
+            "get", f"{_BASE_API_URL}/entitlements/mcstore", ignore_codes=[401]
+        )
 
+        # The response content is empty if the authorization token isn't set or valid
         if not resp.text:
-            raise LoginError("The bearer token is invalid.")
+            raise LoginFailure("The bearer token is invalid.")
 
         if resp.status_code == 401:
             raise MissingMinecraftLicense
@@ -97,7 +103,7 @@ class MojangAuth(HTTPClient):
             raise MissingMinecraftLicense
 
         # This check still needs to be verified
-        resp = self._get(f"{BASE_API_URL}/minecraft/profile")
+        resp = self.request("get", f"{_BASE_API_URL}/minecraft/profile")
         if not resp.ok:
             raise MissingMinecraftProfile
 
@@ -118,7 +124,9 @@ class MojangAuth(HTTPClient):
             "locale": "en",
         }
 
-        resp = self._get("https://login.live.com/oauth20_authorize.srf", params=params)
+        resp = self.request(
+            "get", "https://login.live.com/oauth20_authorize.srf", params=params
+        )
 
         # Parses the values via regex since the HTML can't be parsed
         value = re.search(r'value="(.+?)"', resp.text)[0].replace('value="', "")[:-1]
@@ -135,9 +143,9 @@ class MojangAuth(HTTPClient):
             "PPFT": token,
         }
 
-        resp = self._post(url, data=payload)
+        resp = self.request("post", url, data=payload)
         if "access_token" not in resp.url:
-            raise LoginError
+            raise LoginFailure
 
         raw_login_data = resp.url.split("#")[1]
         data = dict(item.split("=") for item in raw_login_data.split("&"))
@@ -159,8 +167,8 @@ class MojangAuth(HTTPClient):
             "TokenType": "JWT",
         }
 
-        resp = self._post(
-            "https://user.auth.xboxlive.com/user/authenticate", json=json_data
+        resp = self.request(
+            "post", "https://user.auth.xboxlive.com/user/authenticate", json=json_data
         )
 
         xbl_token = resp.json()["Token"]
@@ -168,7 +176,7 @@ class MojangAuth(HTTPClient):
 
         return xbl_token, user_hash
 
-    def _get_xsts_token(self, xbl_token: str):
+    def _get_xsts_token(self, xbl_token):
         """Gets the XSTS token which is required to authenticate with Minecraft services"""
         json_data = {
             "Properties": {"SandboxId": "RETAIL", "UserTokens": [xbl_token]},
@@ -176,27 +184,31 @@ class MojangAuth(HTTPClient):
             "TokenType": "JWT",
         }
 
-        resp = self._post(
-            "https://xsts.auth.xboxlive.com/xsts/authorize", json=json_data
+        resp = self.request(
+            "post",
+            "https://xsts.auth.xboxlive.com/xsts/authorize",
+            ignore_codes=401,
+            json=json_data,
         )
 
         if resp.status_code == 401:
             data = resp.json()
-            if data["XErr"]:
-                if data["XErr"] in _XErrors:
-                    raise LoginError(data["XErr"])
-                raise HTTPException(resp)
+            if data.get("XErr"):
+                if data["XErr"] in _XERRORS:
+                    raise LoginFailure(data["XErr"])
+            raise MojangError(response=resp)
 
         return resp.json()["Token"]
 
-    def _authenticate_with_minecraft(self, user_hash: str, xsts_token: str):
+    def _authenticate_with_minecraft(self, user_hash, xsts_token):
         json_payload = {
             "identityToken": f"XBL3.0 x={user_hash};{xsts_token}",
             "ensureLegacyEnabled": True,
         }
 
-        resp = self._post(
-            f"{BASE_API_URL}/authentication/login_with_xbox",
+        resp = self.request(
+            "post",
+            f"{_BASE_API_URL}/authentication/login_with_xbox",
             json=json_payload,
         )
 
@@ -235,7 +247,7 @@ class Client(MojangAuth):
                 print(skin.variant)
             ```
         """
-        data = self._get(f"{BASE_API_URL}/minecraft/profile").json()
+        data = self.request("get", f"{_BASE_API_URL}/minecraft/profile").json()
 
         capes = []
         skins = []
@@ -287,7 +299,9 @@ class Client(MojangAuth):
             print(name_obj.created_at)
             ```
         """
-        data = self._get(f"{BASE_API_URL}/minecraft/profile/namechange").json()
+        data = self.request(
+            "get", f"{_BASE_API_URL}/minecraft/profile/namechange"
+        ).json()
 
         return NameInformation(
             changed_at=data.get("changedAt"),
@@ -318,7 +332,9 @@ class Client(MojangAuth):
             ```
         """
 
-        resp = self._get(f"{BASE_API_URL}/minecraft/profile/name/{username}/available")
+        resp = self.request(
+            "get", f"{_BASE_API_URL}/minecraft/profile/name/{username}/available"
+        )
 
         return bool(resp.json().get("name"))
 
@@ -339,7 +355,9 @@ class Client(MojangAuth):
         Returns:
             `True` if the username is blocked; `False` if the username is not blocked
         """
-        resp = self._get(f"{BASE_API_URL}/minecraft/profile/name/{username}/available")
+        resp = self.request(
+            "get", f"{_BASE_API_URL}/minecraft/profile/name/{username}/available"
+        )
 
         data = resp.json()
 
@@ -374,26 +392,33 @@ class Client(MojangAuth):
             ```
         """
 
-        resp = self._put(f"{BASE_API_URL}/minecraft/profile/name/{username}")
+        resp = self.request(
+            "put",
+            f"{_BASE_API_URL}/minecraft/profile/name/{username}",
+            ignore_codes=[400, 403],
+        )
 
         if resp.ok:
             return dict(success=True)
 
-        data = resp.json()
-
-        if data.get("errorMessage"):
-            if data["error"] == "FORBIDDEN":
-                return dict(
-                    success=False,
-                    error="This username is already taken or the account does not have an available name change.",
+        if resp.status_code == 400:
+            try:
+                error = resp.json()["errorMessage"].replace(
+                    "changeProfileName.profileName:", ""
                 )
+            except (requests.JSONDecodeError, KeyError):
+                raise BadRequest(response=resp)
 
-            error_message = data["errorMessage"].replace(
-                "changeProfileName.profileName:", ""
+            return dict(success=False, error=error)
+
+        if resp.status_code == 403:
+            return dict(
+                success=False,
+                error="Either the current account does not have an available name change, or the name you supplied"
+                "has already been taken or is still on cooldown.",
             )
-            return dict(success=False, error=error_message)
 
-        raise HTTPException(resp)
+        raise MojangError(response=resp)
 
     def change_skin(
         self,
@@ -429,23 +454,24 @@ class Client(MojangAuth):
 
         if variant != "slim" and variant != "classic":
             raise ValueError("Skin variant must be set to either slim or classic.")
-
-        if not image_path and not url:
+        elif not image_path and not url:
             raise TypeError(
                 "Missing required parameters. Please supply a skin URL or a skin image path."
             )
 
         if url:
             json_payload = {"url": url, "variant": variant}
-            resp = self._post(
-                f"{BASE_API_URL}/minecraft/profile/skins", json=json_payload
+            resp = self.request(
+                "post", f"{_BASE_API_URL}/minecraft/profile/skins", json=json_payload
             )
         else:
             files = {
                 "file": open(f"{image_path}", "rb"),
                 "variant": (None, variant),
             }
-            resp = self._post(f"{BASE_API_URL}/minecraft/profile/skins", files=files)
+            resp = self.request(
+                "post", f"{_BASE_API_URL}/minecraft/profile/skins", files=files
+            )
 
         if not resp.ok:
             error_message = resp.json()["errorMessage"]
@@ -476,16 +502,16 @@ class Client(MojangAuth):
         if not username and not uuid:
             raise TypeError("Either a username or a UUID must be supplied")
         if username:
-            resp = self._get(
-                f"https://api.mojang.com/users/profiles/minecraft/{username}"
+            resp = self.request(
+                "get", f"https://api.mojang.com/users/profiles/minecraft/{username}"
             )
             try:
                 uuid = resp.json()["id"]
             except json.decoder.JSONDecodeError:
                 raise MojangError("Username does not exist")
 
-        resp = self._get(
-            f"https://sessionserver.mojang.com/session/minecraft/profile/{uuid}"
+        resp = self.request(
+            "get", f"https://sessionserver.mojang.com/session/minecraft/profile/{uuid}"
         )
 
         try:
@@ -524,4 +550,4 @@ class Client(MojangAuth):
 
     def reset_skin(self) -> None:
         """Reset your Minecraft skin back to the default one"""
-        self._delete(f"{BASE_API_URL}/minecraft/profile/skins/active")
+        self.request("delete", f"{_BASE_API_URL}/minecraft/profile/skins/active")
